==> deepr-2.1.1.tar.gz: deepr-2.1.1/deepr/layers/slice.py <==
"""Slicing Layers"""

import tensorflow as tf

from deepr.layers import base


class Slice(base.Layer):
    """Slice Layer"""

    def __init__(self, begin: int, end: int, **kwargs):
        super().__init__(n_in=1, n_out=1, **kwargs)
        self.begin = begin
        self.end = end

    def forward(self, tensors, mode: str = None):
        """Forward method of the layer"""
        return tensors[self.begin : self.end]


class SliceFirst(base.Layer):
    """Slice First Layer"""

    def __init__(self, size: int, **kwargs):
        super().__init__(n_in=1, n_out=1, **kwargs)
        self.size = size

    def forward(self, tensors, mode: str = None):
        """Forward method of the layer"""
        return tensors[: self.size]


class SliceLast(base.Layer):
    """Slice First Layer"""

    def __init__(self, size: int, **kwargs):
        super().__init__(n_in=1, n_out=1, **kwargs)
        self.size = size

    def forward(self, tensors, mode: str = None):
        """Forward method of the layer"""
        return tensors[-self.size :]


class SliceLastPadded(base.Layer):
    """Get the values that corresponds to the last not padded values"""

    def __init__(self, padded_value: int = -1, **kwargs):
        super().__init__(n_in=2, n_out=1, **kwargs)
        self.padded_value = padded_value

    def get_length(self, batch):
        positive_values = tf.cast(tf.math.not_equal(batch, self.padded_value), tf.int32)
        real_length = tf.reduce_sum(positive_values, -1)
        real_length = tf.cast(real_length, tf.int32)
        return real_length

    def forward(self, tensors, mode: str = None):
        """Forward method of the layer"""
        vectors, product_indices = tensors

        # 1. Get indices of the last non '-1' value in each timeline
        lengths = self.get_length(product_indices)
        indices = lengths - tf.constant(1, dtype=lengths.dtype)

        # 2. Get coordinates of the last non '-1' value in each timeline (zip `indices` with range(batch_size))
        batch_size = tf.shape(vectors, out_type=tf.int64)[0]
        rang = tf.range(0, batch_size, dtype=tf.int64)

        indices = tf.cast(indices, rang.dtype)
        positions = tf.stack([rang, indices], axis=1)

        return tf.gather_nd(vectors, positions)

==> deepr-2.8.1.tar.gz: deepr-2.8.1/deepr/layers/slice.py <==
"""Slicing Layers"""

import tensorflow as tf

from deepr.layers import base


class Slice(base.Layer):
    """Slice Layer"""

    def __init__(self, begin: int, end: int, **kwargs):
        super().__init__(n_in=1, n_out=1, **kwargs)
        self.begin = begin
        self.end = end

    def forward(self, tensors, mode: str = None):
        """Forward method of the layer"""
        return tensors[self.begin : self.end]


class SliceFirst(base.Layer):
    """Slice First Layer"""

    def __init__(self, size: int, **kwargs):
        super().__init__(n_in=1, n_out=1, **kwargs)
        self.size = size

    def forward(self, tensors, mode: str = None):
        """Forward method of the layer"""
        return tensors[: self.size]


class SliceLast(base.Layer):
    """Slice First Layer"""

    def __init__(self, size: int, **kwargs):
        super().__init__(n_in=1, n_out=1, **kwargs)
        self.size = size

    def forward(self, tensors, mode: str = None):
        """Forward method of the layer"""
        return tensors[-self.size :]


class SliceLastPadded(base.Layer):
    """Get the values that corresponds to the last not padded values"""

    def __init__(self, **kwargs):
        super().__init__(n_in=2, n_out=1, **kwargs)

    def forward(self, tensors, mode: str = None):
        """Forward method of the layer"""
        vectors, mask = tensors
        batch_size, sequence_length = tf.shape(vectors)[0], tf.shape(vectors)[1]
        indices = tf.tile(tf.expand_dims(tf.range(sequence_length), 0), [batch_size, 1])
        indices *= tf.cast(mask, tf.int32)
        lengths = tf.reduce_max(indices, axis=-1)
        positions = tf.stack([tf.range(0, batch_size), lengths], axis=1)
        return tf.gather_nd(vectors, tf.cast(positions, tf.int64))
